import 'dart:convert';
import 'dart:developer' as developer;

import 'package:dio/dio.dart';

class DioLogInterceptor extends Interceptor {
  void Function(Object object) logPrint = (s) {
    bool kReleaseMode = bool.fromEnvironment('dart.vm.product', defaultValue: false);
    bool kProfileMode = bool.fromEnvironment('dart.vm.profile', defaultValue: false);
    if (kReleaseMode || kProfileMode) return;
    developer.log(s.toString());
  };

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) {
    logPrint('****************** Request ******************');
    _printKV('uri', options.uri);
    _printKV('method', options.method);
    _printKV('responseType', options.responseType.toString());
    logPrint('headers:');
    options.headers.forEach((key, v) => _printKV(' $key', v));
    if ("GET" == options.method) {
      logPrint('queryParameters:');
      StringBuffer queryParameters = StringBuffer();
      var queryParametersLength = options.queryParameters.keys.length;
      for (int i = 0; i < options.queryParameters.keys.length; i++) {
        var k = options.queryParameters.keys.elementAt(i);
        var v = options.queryParameters.values.elementAt(i);
        queryParameters.write(k);
        queryParameters.write("=");
        queryParameters.write(v);
        if (i != queryParametersLength - 1) {
          queryParameters.write("&");
        }
      }
      logPrint(queryParameters);
    } else if ("POST" == options.method) {
      if (options.data != null) {
        try {
          if (options.data is FormData) {
            var map = {};
            for (var value in (options.data as FormData).fields) {
              if (value.value is MultipartFile) {
                map[value.key] = (value.value as MultipartFile).filename;
              } else {
                map[value.key] = value.value;
              }
            }
            logPrint('requestBody:');
            logPrint(json.encode(map));
          }
        } catch (e) {
          print(e);
        }
      }
    }
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) async {
    logPrint('*** Response ***');
    _printResponse(response);
    handler.next(response);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) async {
    if (err.type != DioErrorType.cancel) {
      logPrint('*** DioError ***:');
      logPrint('uri: ${err.requestOptions.uri}');
      logPrint('$err');
      if (err.response != null) {
        _printResponse(err.response!);
      }
      logPrint('');
      handler.next(err);
    } else {
      logPrint('Dio cancel  uri:  ${err.requestOptions.uri}');
    }
  }

  void _printResponse(Response response) {
    _printKV('uri', response.requestOptions.uri);
    if (!response.headers.isEmpty) {
      _printKV('statusCode', response.statusCode);
      if (response.isRedirect == true) {
        _printKV('redirect', response.realUri);
      }

      logPrint('headers:');
      response.headers.forEach((key, v) => _printKV(' $key', v.join('\r\n\t')));
    }
    logPrint('Response Text:');
    _printAll(response.toString());
    logPrint('');
  }

  void _printKV(String key, Object? v) => logPrint('$key: $v');

  void _printAll(msg) => msg.toString().split('\n').forEach(logPrint);
}
