library itwo_oss_post;

import 'dart:convert';

import 'package:crypto/crypto.dart';
import 'package:dio/dio.dart';

class OSSUtil {
  ///  [credentials] 服务端返回的 oss 签名信息
  ///  [config] oss 的配置信息
  ///  [file]  上传的文件
  ///  [fileName] 保存到 oss 上的全路径的文件名 (test/user/head.png)
  ///  [callbackParam] 上传到 oss 的回调
  ///  上传的图片可以用 http://image-demo.oss-cn-hangzhou.aliyuncs.com/example.jpg?x-oss-process=image/info 查询图片信息

  static Future<OSSResult> postObject(Credentials credentials, OSSConfig config, String filePath, String fileName, {CancelToken? cancelToken, ProgressCallback? progressCallback, Map? callbackParam}) async {
    String policyText = '{"expiration": "${credentials.expiration}","conditions": [{"bucket": "${config.bucket}" },["content-length-range", 0, 1048576000]]}';
    // 获取签名
    String signature = _getSignature(credentials, policyText);

    BaseOptions options = new BaseOptions();

    options.responseType = ResponseType.plain;
    if (callbackParam != null) {
      options.headers = {"x-oss-callback": _getBase64Str(json.encoder.convert(callbackParam))};
    }
    //创建dio对象
    Dio dio = new Dio(options);

    // 请求参数的form对象
    var map = {
      'key': fileName,
      'policy': _getBase64Str(policyText),
      'OSSAccessKeyId': credentials.accessKeyId,
      'success_action_status': '200', //让服务端返回200，不然，默认会返回204
      'signature': signature,
      'contentType': 'multipart/form-data',
      'file': MultipartFile.fromFileSync(filePath),
      "x-oss-security-token":credentials.securityToken,
    };

    FormData data = new FormData.fromMap(map);
    try {
      // var url ="${config.url}?callback=${_getBase64Str(json.encoder.convert(callbackParam))}";
      // 发送请求
      var response = await dio.post(config.host, data: data, cancelToken: cancelToken, onSendProgress: progressCallback);
      return OSSResult(200, host: config.host, fileName: fileName);
    } catch (e) {
      Response? response = (e as DioError).response;
      return OSSResult(response?.statusCode ?? -1, message: response?.toString() ?? "");
    }
  }

  // 获取plice的base64
  static _getBase64Str(String str) {
    //进行utf8编码
    List<int> str_utf8 = utf8.encode(str);
    //进行base64编码
    String str_base64 = base64.encode(str_utf8);
    return str_base64;
  }

  /// 获取签名
  static String _getSignature(Credentials credentials, String policyText) {
    //进行utf8编码
    List<int> policyText_utf8 = utf8.encode(policyText);
    //进行base64编码
    String policy_base64 = base64.encode(policyText_utf8);
    //再次进行utf8编码
    List<int> policy = utf8.encode(policy_base64);
    //进行utf8 编码
    List<int> key = utf8.encode(credentials.accessKeySecret);
    //通过hmac,使用sha1进行加密
    List<int> signature_pre = Hmac(sha1, key).convert(policy).bytes;
    //最后一步，将上述所得进行base64 编码
    String signature = base64.encode(signature_pre);
    return signature;
  }
}

class Credentials {
  String accessKeyId;
  String accessKeySecret;
  String expiration;
  String securityToken;

  Credentials(this.accessKeyId, this.accessKeySecret, this.expiration, this.securityToken);
}

class OSSConfig {
  String bucket;
  String endpoint;

  OSSConfig(this.bucket, this.endpoint);

  String get host => "http://$bucket.$endpoint/";
}

class OSSResult {
  final int code;
  final String? fileName;
  final String? host;
  final String? message;

  const OSSResult(this.code, {this.host, this.fileName, this.message});
}
